package minischeme

abstract class Instruction(val opcode: Opcode) extends Representable {
  def representation: String =
    "\t" + opcode + " " + (args map { x => x.representation}).mkString(" ")
  protected def args: List[Representable]
}

class InstructionRRR(_op: OpcodeRRR, _r1: Register, _r2: Register, _r3: Register) extends Instruction(_op) {
  var r1: Register = _r1
  var r2: Register = _r2
  var r3: Register = _r3
  override def args: List[Representable] = List(r1,r2,r3)
}

class InstructionRR(_op: OpcodeRR, _r1: Register, _r2: Register) extends Instruction(_op) {
  var r1: Register = _r1
  var r2: Register = _r2
  override def args: List[Representable] = List(r1,r2)
}

class InstructionRC(_op: LINT.type, _r: Register, _c: Representable) extends Instruction(_op) {
  var r: Register = _r
  var c: Representable = _c
  override def args: List[Representable] = List(r, c)
}

class InstructionR(_op: OpcodeR, _r: Register) extends Instruction(_op) {
  var r: Register = _r
  override def args: List[Representable] = List(r)
}

class InstructionHalt(_op: HALT.type) extends Instruction(_op) {
  override def args: List[Representable] = List()
}
