package minischeme

import java.io.{FileInputStream,FileOutputStream,PrintStream,FileNotFoundException}
import scala.collection.immutable.ListMap

abstract class MainCommon {
  val globalEnv =
    ListMap(Primitives.names map {p => (p, new Symbol(p, PrimitiveKind))}: _*)

  def usageString: String
  def error(msg: String): Nothing = {
    Console.println("Usage: scala minischeme." + usageString)
    Console.println("\nError: " + msg)
    exit(1)
  }

  def parseInputFiles(inFiles: List[String]): List[Tree] = {
    if (inFiles.isEmpty)
      error("no input files given")
    try {
      val inStreams = inFiles map (new FileInputStream(_))
      val program = (new Parser(new Scanner(inFiles zip inStreams))).program()
      NameAnalyzer(globalEnv, program)
      EtaExpander(program)
    } catch {
      case e : FileNotFoundException => error(e.getMessage)
    }
  }
}

object MainInterpreter extends MainCommon {
  override def usageString: String = "MainInterpreter <input_file> ..."

  def main(args: Array[String]): Unit =
    Interpreter(parseInputFiles(args.toList))
}

object MainCompiler extends MainCommon {
  override def usageString: String =
    "MainCompiler [-o <output_file>] <input_file> ..."

  def main(args: Array[String]): Unit = {
    try {
      val (inFiles, outStream) = {
        def loop(args: List[String]): (List[String], PrintStream) =
          args match {
            case "-o" :: outFile :: rest =>
              (rest, new PrintStream(new FileOutputStream(outFile)))
            case arg :: rest =>
              val (in, out) = loop(rest);
              (arg :: in, out)
            case Nil =>
              (Nil, System.out)
          }
        loop(args.toList)
      }

      Generator(parseInputFiles(inFiles), outStream)
    } catch {
      case e: FileNotFoundException => error(e.getMessage)
    }
  }
}
