package minischeme

/**
 * Utility functions to build different kinds of trees.
 *
 * @author Michel Schinz
 */
object TreeGenerator {
  def slist(args: Tree*): Tree =
    SList(args.toList)

  def define(name: String, value: Tree): Tree =
    SList(SIdent("define") :: SIdent(name) :: value :: Nil)

  def lambda(params: List[Tree], body: List[Tree]): Tree =
    SList(SIdent("lambda") :: SList(params) :: body)

  def let(sym: Symbol, value: Tree, body: List[Tree]): Tree =
    SList(SIdent("let")
          :: SList(List(SList(List(ident(sym), value))))
          :: body)

  def ident(symbol: Symbol): Tree = {
    val tree = SIdent(symbol.name)
    tree.symbol = symbol
    tree
  }

  def call(funAndArgs: Tree*): Tree =
    SList(funAndArgs.toList)

  def vector(vs: List[Tree]): Tree =
    SList(SIdent("vector") :: vs)

  def vectorRef(block: Tree, index: Int): Tree =
    call(SIdent("vector-ref"), block, SNum(index))
}
